#version 330

#pragma region [SHADER]

    // Input vertex attributes
    in vec3 vertexPosition;
    in vec2 vertexTexCoord;
    in vec4 vertexColor;
    in vec3 vertexNormal;
    in vec4 vertexTangent;

    // Output
    out vec4 fragPosition;
    out vec3 fragPosWorld;
    out vec4 fragPosView;
    out vec2 fragTexCoord;
    out vec4 fragColor;
    out vec3 fragNormal;
    out vec3 fragNormalView;
    out vec3 fragTangent;
    out vec3 fragBitangent;

    // Variables
    uniform mat4 mvp;
    uniform mat4 matNormal;
    uniform mat4 matModel;
    uniform mat4 matView;
    uniform mat4 matProj;

#pragma endregion

void main()
{
    
    // Send vertex attributes to fragment shader
    vec4 tmpPosWorld = matModel * vec4(vertexPosition, 1.0);
    fragPosWorld  = tmpPosWorld.xyz;
    fragPosView   = matView * tmpPosWorld;
    
    fragPosition  = matProj * fragPosView;

    fragTexCoord  = vertexTexCoord;
    fragColor     = vertexColor;

    mat3 normalMatrix = transpose(inverse(mat3(matModel)));
    fragNormal = normalize(normalMatrix * vertexNormal);
    fragNormalView = normalize(mat3(matView) * normalMatrix * vertexNormal).xyz;
    
    // Check if the vertexTangent is zero (i.e., model doesn't provide tangents)
    if (length(vertexTangent.xyz) > 0.0) {
        fragTangent = normalize(normalMatrix * vertexTangent.xyz);
        fragBitangent = cross(fragNormal, fragTangent) * vertexTangent.w; // Normal Map: Overlapping support
    } else {
        // Generate a default tangent when none is provided
        vec3 up = vec3(0.0, 1.0, 0.0);
        vec3 right = vec3(1.0, 0.0, 0.0);

        // If the normal is too close to the up vector, use the right vector to generate tangents
        if (abs(dot(fragNormal, up)) > 0.999) {
            fragTangent = normalize(cross(fragNormal, right));
        } else {
            fragTangent = normalize(cross(fragNormal, up));
        }

        // Assume w = 1 for default tangents when vertexTangent is not provided
        fragBitangent = cross(fragNormal, fragTangent);  // No overlapping without valid w
    }

    // Calculate final vertex position
    gl_Position = matProj * matView * matModel * vec4(vertexPosition, 1.0);

}